diff --git a/debian/initramfs/cryptroot-script b/debian/initramfs/cryptroot-script
index 8c40af5..af0e772 100644
--- a/debian/initramfs/cryptroot-script
+++ b/debian/initramfs/cryptroot-script
@@ -303,7 +303,7 @@ setup_mapping()
 				return 1
 			fi
 
-			NEWROOT="/dev/mapper/$cryptlvm"
+			NEWROOT=${cmdline_root:-/dev/mapper/$cryptlvm}
 			if [ "$cryptrootdev" = "yes" ]; then
 				# required for lilo to find the root device
 				echo "ROOT=$NEWROOT" >> /conf/param.conf
@@ -336,18 +336,38 @@ setup_mapping()
 #
 
 # Do we have any kernel boot arguments?
-found=''
+cmdline_cryptopts=''
+unset cmdline_root
 for opt in $(cat /proc/cmdline); do
 	case $opt in
 	cryptopts=*)
-		found=yes
-		setup_mapping "${opt#cryptopts=}"
+      	        opt="${opt#cryptopts=}"
+		if [ -n "$opt" ]; then
+		    if [ -n "$cmdline_cryptopts" ]; then
+			cmdline_cryptopts="$cmdline_cryptopts $opt"
+		    else
+			cmdline_cryptopts="$opt"
+		    fi
+		fi
 		;;
+        root=*)
+	        opt="${opt#root=}"
+		case $opt in
+		    /*) # Absolute path given. Not lilo major/minor number.
+			cmdline_root=$opt
+			;;
+		    *) # lilo major/minor number (See #398957). Ignore
+		esac
+	        ;;
 	esac
 done
 
-if [ -n "$found" ]; then
-	exit 0
+if [ -n "$cmdline_cryptopts" ]; then
+    # Call setup_mapping separately for each possible cryptopts= setting
+    for cryptopt in $cmdline_cryptopts; do
+	setup_mapping "$cryptopt"
+    done
+    exit 0
 fi
 
 # Do we have any settings from the /conf/conf.d/cryptroot file?
